<?php
/**
 * @wordpress-plugin
 * Plugin Name: Visidea
 * Plugin URI: https://visidea.ai
 * Description: Visidea is the visual search and visual recommendation plugin for WooCommerce. Visidea improves UX and increases the revenues of your website.
 * Version: 1.2.9
 * Author: Inferendo
 * Author URI: https://visidea.ai
 * License: GPLv3
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 * Text Domain: visidea
 * @package Visidea
 */

// If this file is called directly, abort.
if (!defined( 'WPINC')) {
	die;
}

/**
 * Require action scheduler.
 */
require_once( plugin_dir_path( __FILE__ ) . 'libraries/action-scheduler/action-scheduler.php' );

/**
 * Current Visidea version.
 */
define('VISIDEA_VERSION', '1.2.9');

/**
 * The code that runs during Visidea activation.
 * This action is documented in includes/class-visidea-activator.php
 */
function activate_visidea() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-visidea-activator.php';
    Visidea_Activator::activate();
}
register_activation_hook( __FILE__, 'activate_visidea' );

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-plugin-name-deactivator.php
 */
function deactivate_visidea() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-visidea-deactivator.php';
	Visidea_Deactivator::deactivate();
}
register_deactivation_hook( __FILE__, 'deactivate_visidea' );

/**
 * The core Visidea class that is used to define admin-specific hooks
 * and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-visidea.php';

/**
 * Begins execution of Visidea.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_visidea() {
	$plugin = new Visidea();
	$plugin->run();
}

/**
 * A callback to run when the scheduled action is run.
 */
function visidea_export_data_job() {
// 	$plugin_admin = new Visidea_Admin('visidea', VISIDEA_VERSION);
// 	$plugin_admin->export_data();
}
add_action( 'visidea_export_data_job', 'visidea_export_data_job' );

function visidea_export_items_job() {
	$plugin_admin = new Visidea_Admin('visidea', VISIDEA_VERSION);
	$plugin_admin->cron_items_dump();
}
add_action( 'visidea_export_items_job', 'visidea_export_items_job' );

function visidea_export_users_job() {
	$plugin_admin = new Visidea_Admin('visidea', VISIDEA_VERSION);
	$plugin_admin->cron_users_dump();
}
add_action( 'visidea_export_users_job', 'visidea_export_users_job' );

function visidea_export_interactions_job() {
	$plugin_admin = new Visidea_Admin('visidea', VISIDEA_VERSION);
	$plugin_admin->cron_interactions_dump();
}
add_action( 'visidea_export_interactions_job', 'visidea_export_interactions_job' );

run_visidea();
